@@ -1,5 +1,5 @@
 // BadWolf: Minimalist and privacy-oriented WebKitGTK+ browser
-// SPDX-FileCopyrightText: 2019-2023 Badwolf Authors <https://hacktivis.me/projects/badwolf>
+// SPDX-FileCopyrightText: 2019-2024 Badwolf Authors <https://hacktivis.me/projects/badwolf>
 // SPDX-License-Identifier: BSD-3-Clause
 
 #include "badwolf.h"
@@ -18,15 +18,19 @@
 #include <locale.h>       /* LC_* */
 #include <stdio.h>        /* perror(), fprintf(), snprintf() */
 #include <stdlib.h>       /* malloc() */
-#include <unistd.h>       /* access() */
+#include <unistd.h>       /* access(), getopt() */
 
 const gchar *homepage = "https://hacktivis.me/projects/badwolf";
 const gchar *version  = VERSION;
 
+struct Window *window = NULL;
+
 static gchar *web_extensions_directory;
 static uint64_t context_id_counter = 0;
 GtkTreeModel *bookmarks_completion_model;
 
+bool opt_S = false, opt_i = false;
+
 static gboolean WebViewCb_close(WebKitWebView *webView, gpointer user_data);
 static gboolean WebViewCb_web_process_terminated(WebKitWebView *webView,
                                                  WebKitWebProcessTerminationReason reason,
@@ -104,6 +108,10 @@ WebViewCb_web_process_terminated(WebKitWebView *UNUSED(webView),
 		fprintf(stderr, "%s", _("the web process exceeded the memory limit.\n"));
 		webView_tab_label_change(browser, _("Out of Memory"));
 		break;
+	case WEBKIT_WEB_PROCESS_TERMINATED_BY_API:
+		fprintf(stderr, "%s", _("the web process was user terminated.\n"));
+		webView_tab_label_change(browser, _("User terminated"));
+		break;
 	default:
 		fprintf(stderr, "%s", _("the web process terminated for an unknown reason.\n"));
 		webView_tab_label_change(browser, _("Unknown Crash"));
@@ -232,7 +240,7 @@ WebViewCb_notify__is__playing__audio(WebKitWebView *UNUSED(webView),
 void
 webView_tab_label_change(struct Client *browser, const gchar *title)
 {
-	GtkWidget *notebook = browser->window->notebook;
+	GtkWidget *notebook = window->notebook;
 
 #define title_IS_EMPTY title == NULL || title[0] == '\0'
 
@@ -247,7 +255,7 @@ webView_tab_label_change(struct Client *browser, const gchar *title)
 	// Set the window title if the title change was on the current tab
 	if(gtk_notebook_page_num(GTK_NOTEBOOK(notebook), browser->box) ==
 	   gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook)))
-		gtk_window_set_title(GTK_WINDOW(browser->window->main_window), title);
+		gtk_window_set_title(GTK_WINDOW(window->main_window), title);
 }
 
 static gboolean
@@ -317,9 +325,9 @@ WebViewCb_create(WebKitWebView *related_web_view,
 	// shouldn't be needed but better be safe
 	old_browser->webView = related_web_view;
 
-	browser = new_browser(old_browser->window, NULL, old_browser);
+	browser = new_browser(NULL, old_browser);
 
-	if(badwolf_new_tab(GTK_NOTEBOOK(old_browser->window->notebook), browser, FALSE) < 0)
+	if(badwolf_new_tab(GTK_NOTEBOOK(window->notebook), browser, FALSE) < 0)
 		return NULL;
 	else
 		return browser->webView;
@@ -357,9 +365,17 @@ WebViewCb_decide_policy(WebKitWebView *UNUSED(web_view),
 		{
 			WebKitURIRequest *uri   = webkit_navigation_action_get_request(navigation_action);
 			const gchar *target_url = webkit_uri_request_get_uri(uri);
-			struct Client *browser  = new_browser(old_browser->window, target_url, old_browser);
+			struct Client *browser  = new_browser(target_url, old_browser);
+			int ret                 = badwolf_new_tab(GTK_NOTEBOOK(window->notebook), browser, FALSE);
 
-			badwolf_new_tab(GTK_NOTEBOOK(browser->window->notebook), browser, FALSE);
+			if(ret < 0)
+			{
+				fprintf(stderr,
+				        "badwolf: Error while opening <%s> in a new tab (badwolf_new_tab returned %d)\n",
+				        target_url,
+				        ret);
+				return FALSE;
+			}
 			webkit_web_view_load_uri(browser->webView, target_url);
 			webkit_policy_decision_ignore(decision);
 		}
@@ -446,7 +462,7 @@ WebViewCb_load_failed_with_tls_errors(WebKitWebView *UNUSED(web_view),
 	SoupURI *failing_uri = soup_uri_new(failing_text);
 #endif
 
-	GtkWidget *dialog = gtk_message_dialog_new(GTK_WINDOW(browser->window->main_window),
+	GtkWidget *dialog = gtk_message_dialog_new(GTK_WINDOW(window->main_window),
 	                                           GTK_DIALOG_MODAL & GTK_DIALOG_DESTROY_WITH_PARENT,
 	                                           GTK_MESSAGE_ERROR,
 	                                           GTK_BUTTONS_NONE,
@@ -500,15 +516,12 @@ web_contextCb_download_started(WebKitWebContext *UNUSED(web_context),
                                WebKitDownload *webkit_download,
                                gpointer user_data)
 {
-	struct Client *browser    = (struct Client *)user_data;
 	struct Download *download = malloc(sizeof(struct Download));
 
 	assert(webkit_download);
 
 	if(download != NULL)
 	{
-		download->window = browser->window;
-
 		download_new_entry(webkit_download, download);
 
 		g_signal_connect(
@@ -592,7 +605,7 @@ printCb_clicked(GtkButton *UNUSED(print), gpointer user_data)
 
 	WebKitPrintOperation *print_operation = webkit_print_operation_new(browser->webView);
 
-	webkit_print_operation_run_dialog(print_operation, GTK_WINDOW(browser->window->main_window));
+	webkit_print_operation_run_dialog(print_operation, GTK_WINDOW(window->main_window));
 }
 
 static gboolean
@@ -648,7 +661,7 @@ widgetCb_drop_button3_event(GtkWidget *UNUSED(widget), GdkEvent *event, gpointer
 }
 
 struct Client *
-new_browser(struct Window *window, const gchar *target_url, struct Client *old_browser)
+new_browser(const gchar *target_url, struct Client *old_browser)
 {
 	struct Client *browser = malloc(sizeof(struct Client));
 	target_url             = badwolf_ensure_uri_scheme(target_url, (old_browser == NULL));
@@ -658,7 +671,8 @@ new_browser(struct Window *window, const gchar *target_url, struct Client *old_b
 
 	if(browser == NULL) return NULL;
 
-	browser->window     = window;
+	assert(window != NULL);
+
 	browser->context_id = old_browser == NULL ? context_id_counter++ : old_browser->context_id;
 	browser->box        = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	gtk_widget_set_name(browser->box, "browser__box");
@@ -725,9 +739,14 @@ new_browser(struct Window *window, const gchar *target_url, struct Client *old_b
 
 			webkit_web_context_set_spell_checking_enabled(web_context, TRUE);
 		}
+
+		webkit_web_context_register_uri_scheme(
+		    web_context, "man", man_uri_scheme_request_cb, NULL, NULL);
 	}
 
 	WebKitSettings *settings = webkit_settings_new_with_settings(BADWOLF_WEBKIT_SETTINGS);
+	if(opt_S) webkit_settings_set_enable_javascript_markup(settings, true);
+	if(opt_i) webkit_settings_set_auto_load_images(settings, false);
 
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(browser->javascript),
 	                             webkit_settings_get_enable_javascript_markup(settings));
@@ -835,7 +854,7 @@ new_browser(struct Window *window, const gchar *target_url, struct Client *old_b
 	g_signal_connect(
 	    browser->forward, "button-release-event", G_CALLBACK(widgetCb_drop_button3_event), NULL);
 
-	/* signals for javacript toggle widget */
+	/* signals for javascript toggle widget */
 	g_signal_connect(browser->javascript, "toggled", G_CALLBACK(javascriptCb_toggled), browser);
 	/* prevents GtkNotebook from spawning it's context-menu */
 	g_signal_connect(
@@ -950,10 +969,9 @@ badwolf_new_tab(GtkNotebook *notebook, struct Client *browser, bool auto_switch)
 }
 
 static void
-new_tabCb_clicked(GtkButton *UNUSED(new_tab), gpointer user_data)
+new_tabCb_clicked(GtkButton *UNUSED(new_tab), gpointer UNUSED(user_data))
 {
-	struct Window *window  = (struct Window *)user_data;
-	struct Client *browser = new_browser(window, NULL, NULL);
+	struct Client *browser = new_browser(NULL, NULL);
 
 	badwolf_new_tab(GTK_NOTEBOOK(window->notebook), browser, TRUE);
 }
@@ -970,20 +988,18 @@ static void
 notebookCb_switch__page(GtkNotebook *notebook,
                         GtkWidget *page,
                         guint UNUSED(page_num),
-                        gpointer user_data)
+                        gpointer UNUSED(user_data))
 {
-	struct Window *window = (struct Window *)user_data;
-	GtkWidget *label      = gtk_notebook_get_tab_label(notebook, page);
+	GtkWidget *label = gtk_notebook_get_tab_label(notebook, page);
 
 	// TODO: Maybe find a better way to store the title
 	gtk_window_set_title(GTK_WINDOW(window->main_window), gtk_widget_get_tooltip_text(label));
 }
 
 void
-content_managerCb_ready(GObject *UNUSED(store), GAsyncResult *result, gpointer user_data)
+content_managerCb_ready(GObject *UNUSED(store), GAsyncResult *result, gpointer UNUSED(user_data))
 {
-	struct Window *window = (struct Window *)user_data;
-	GError *err           = NULL;
+	GError *err = NULL;
 
 	WebKitUserContentFilter *filter =
 	    webkit_user_content_filter_store_load_finish(window->content_store, result, &err);
@@ -1012,10 +1028,9 @@ content_managerCb_ready(GObject *UNUSED(store), GAsyncResult *result, gpointer u
 static void
 storeCb_finish(WebKitUserContentFilterStore *UNUSED(store),
                GAsyncResult *result,
-               gpointer user_data)
+               gpointer UNUSED(user_data))
 {
-	struct Window *window = (struct Window *)user_data;
-	GError *err           = NULL;
+	GError *err = NULL;
 
 	WebKitUserContentFilter *filter =
 	    webkit_user_content_filter_store_save_finish(window->content_store, result, &err);
@@ -1047,9 +1062,14 @@ storeCb_finish(WebKitUserContentFilterStore *UNUSED(store),
 int
 main(int argc, char *argv[])
 {
-	struct Window *window = &(struct Window){NULL, NULL, NULL, NULL, NULL, NULL};
 	GApplication *application;
+	errno = 0;
 	setlocale(LC_ALL, "");
+	if(errno != 0)
+	{
+		fprintf(stderr, "badwolf: Warning: Failed to initialize locales: %s\n", strerror(errno));
+		errno = 0;
+	}
 	bindtextdomain(PACKAGE, DATADIR "/locale");
 	bind_textdomain_codeset(PACKAGE, "UTF-8");
 	textdomain(PACKAGE);
@@ -1062,6 +1082,26 @@ main(int argc, char *argv[])
 
 	gtk_init(&argc, &argv);
 
+	int c = EOF;
+	while((c = getopt(argc, argv, "iS")) != EOF)
+	{
+		switch(c)
+		{
+		case 'i':
+			opt_i = true;
+			break;
+		case 'S':
+			opt_S = true;
+			break;
+		case '?':
+			fprintf(stderr, _("badwolf: Unrecognized option: '-%c'\n"), optopt);
+			return 1;
+		}
+	}
+
+	argc -= optind;
+	argv += optind;
+
 	fprintf(stderr, _("Running Badwolf version: %s\n"), version);
 	fprintf(stderr,
 	        _("Buildtime WebKit version: %d.%d.%d\n"),
@@ -1081,22 +1121,26 @@ main(int argc, char *argv[])
 	bookmarks_completion_model = bookmarks_completion_init();
 	g_object_ref(bookmarks_completion_model);
 
-	window->main_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-	window->notebook    = gtk_notebook_new();
-	window->new_tab = gtk_button_new_from_icon_name("tab-new-symbolic", GTK_ICON_SIZE_SMALL_TOOLBAR);
-	window->downloads_tab   = badwolf_downloads_tab_new();
-	window->content_manager = webkit_user_content_manager_new();
+	gchar *filtersPath = g_build_filename(g_get_user_cache_dir(), g_get_prgname(), "filters", NULL);
+
+	window = &(struct Window){
+	    .main_window = gtk_window_new(GTK_WINDOW_TOPLEVEL),
+	    .notebook    = gtk_notebook_new(),
+	    .new_tab     = gtk_button_new_from_icon_name("tab-new-symbolic", GTK_ICON_SIZE_SMALL_TOOLBAR),
+	    .downloads_tab   = badwolf_downloads_tab_new(),
+	    .content_manager = webkit_user_content_manager_new(),
+	    .content_store   = webkit_user_content_filter_store_new(filtersPath),
+	};
 
 	load_userscripts(window->content_manager);
 
+	assert(window != NULL);
+
 	gchar *contentFilterPath =
 	    g_build_filename(g_get_user_config_dir(), g_get_prgname(), "content-filters.json", NULL);
 	GFile *contentFilterFile = g_file_new_for_path(contentFilterPath);
 	fprintf(stderr, _("content-filters file set to: %s\n"), contentFilterPath);
 
-	gchar *filtersPath = g_build_filename(g_get_user_cache_dir(), g_get_prgname(), "filters", NULL);
-	window->content_store = webkit_user_content_filter_store_new(filtersPath);
-
 	webkit_user_content_filter_store_save_from_file(window->content_store,
 	                                                "a",
 	                                                contentFilterFile,
@@ -1146,23 +1190,23 @@ main(int argc, char *argv[])
 	gtk_container_add(GTK_CONTAINER(window->main_window), window->notebook);
 	gtk_widget_queue_draw(window->notebook);
 
-	badwolf_downloads_tab_attach(window);
+	badwolf_downloads_tab_attach();
 
 	g_signal_connect(
-	    window->main_window, "key-press-event", G_CALLBACK(main_windowCb_key_press_event), window);
+	    window->main_window, "key-press-event", G_CALLBACK(main_windowCb_key_press_event), NULL);
 
 	g_signal_connect(window->main_window, "destroy", G_CALLBACK(gtk_main_quit), NULL);
-	g_signal_connect(window->new_tab, "clicked", G_CALLBACK(new_tabCb_clicked), window);
-	g_signal_connect(window->notebook, "switch-page", G_CALLBACK(notebookCb_switch__page), window);
+	g_signal_connect(window->new_tab, "clicked", G_CALLBACK(new_tabCb_clicked), NULL);
+	g_signal_connect(window->notebook, "switch-page", G_CALLBACK(notebookCb_switch__page), NULL);
 
 	gtk_widget_show(window->new_tab);
 	gtk_widget_show_all(window->main_window);
 
-	if(argc == 1)
-		badwolf_new_tab(GTK_NOTEBOOK(window->notebook), new_browser(window, NULL, NULL), FALSE);
+	if(argc == 0)
+		badwolf_new_tab(GTK_NOTEBOOK(window->notebook), new_browser(NULL, NULL), FALSE);
 	else
-		for(int i = 1; i < argc; ++i)
-			badwolf_new_tab(GTK_NOTEBOOK(window->notebook), new_browser(window, argv[i], NULL), FALSE);
+		for(int i = 0; i < argc; ++i)
+			badwolf_new_tab(GTK_NOTEBOOK(window->notebook), new_browser(argv[i], NULL), FALSE);
 
 	gtk_notebook_set_current_page(GTK_NOTEBOOK(window->notebook), 1);
 
