@@ -109,4 +109,9 @@
 // BADWOLF_LOCATION_INLINE_SELECTION: show selected completion as a selection in location entry
 #define BADWOLF_LOCATION_INLINE_SELECTION TRUE
 
+// BADWOLF_MAN_HTML_CMD: Command to render manpages to HTML.
+// Needs to support `[section] name` as additional arguments.
+// Default one made for mandoc
+#define BADWOLF_MAN_HTML_CMD "man", "-Thtml", "-Oman=man:%N.%S", "-Otoc"
+
 #endif /* CONFIG_H_INCLUDED */
