@@ -7,8 +7,6 @@
 
 struct Download
 {
-	struct Window *window;
-
 	GtkWidget *container;
 	GtkWidget *icon;
 	GtkWidget *stop_icon;
@@ -28,4 +26,4 @@ void downloadCb_failed(WebKitDownload *webkit_download, GError *error, gpointer
 void downloadCb_finished(WebKitDownload *download, gpointer user_data);
 void downloadCb_received_data(WebKitDownload *download, guint64 data_lenght, gpointer user_data);
 GtkWidget *badwolf_downloads_tab_new();
-void badwolf_downloads_tab_attach(struct Window *window);
+void badwolf_downloads_tab_attach();
