@@ -1,5 +1,5 @@
 // BadWolf: Minimalist and privacy-oriented WebKitGTK+ browser
-// SPDX-FileCopyrightText: 2019-2022 Badwolf Authors <https://hacktivis.me/projects/badwolf>
+// SPDX-FileCopyrightText: 2019-2023 Badwolf Authors <https://hacktivis.me/projects/badwolf>
 // SPDX-License-Identifier: BSD-3-Clause
 
 #include "keybindings.h"
@@ -9,11 +9,9 @@
 #include <glib/gi18n.h> /* _() */
 
 static gboolean
-about_dialogCb_activate_link(GtkAboutDialog *about_dialog, gchar *uri, gpointer user_data)
+about_dialogCb_activate_link(GtkAboutDialog *about_dialog, gchar *uri, gpointer UNUSED(user_data))
 {
-	struct Window *window = (struct Window *)user_data;
-
-	badwolf_new_tab(GTK_NOTEBOOK(window->notebook), new_browser(window, uri, NULL), TRUE);
+	badwolf_new_tab(GTK_NOTEBOOK(window->notebook), new_browser(uri, NULL), TRUE);
 
 	gtk_window_close(GTK_WINDOW(about_dialog));
 
@@ -21,9 +19,8 @@ about_dialogCb_activate_link(GtkAboutDialog *about_dialog, gchar *uri, gpointer
 }
 
 static void
-badwolf_about_dialog(GtkWindow *main_window, gpointer user_data)
+badwolf_about_dialog(GtkWindow *main_window, gpointer UNUSED(user_data))
 {
-	struct Window *window   = (struct Window *)user_data;
 	GtkWidget *about_dialog = gtk_about_dialog_new();
 
 	char *comments = NULL;
@@ -37,8 +34,9 @@ badwolf_about_dialog(GtkWindow *main_window, gpointer user_data)
 	gtk_window_set_transient_for(GTK_WINDOW(about_dialog), main_window);
 	gtk_window_set_destroy_with_parent(GTK_WINDOW(about_dialog), TRUE);
 
-	gtk_about_dialog_set_license(GTK_ABOUT_DIALOG(about_dialog),
-	                             "SPDX License Identifiers: BSD-3-Clause AND CC-BY-SA-4.0");
+	gtk_about_dialog_set_license(
+	    GTK_ABOUT_DIALOG(about_dialog),
+	    "SPDX License Identifiers: BSD-3-Clause AND CC-BY-SA-4.0 AND MPL-2.0");
 	gtk_about_dialog_set_copyright(
 	    GTK_ABOUT_DIALOG(about_dialog),
 	    "2019-2021 Badwolf Authors <https://hacktivis.me/projects/badwolf>");
@@ -66,7 +64,7 @@ toggle_caret_browsing(WebKitWebView *webView)
 
 /* commonCb_key_press_event: Global callback for keybindings
  *
- * Theses shortcuts should be avoided as much as possible:
+ * These shortcuts should be avoided as much as possible:
  * - Single key shortcuts (ie. backspace and space)
  * - Triple key shortcuts (except for Ctrl+Shift)
  * - Unix Terminal shortcuts (specially Ctrl-W)
@@ -74,7 +72,7 @@ toggle_caret_browsing(WebKitWebView *webView)
  * loosely follows https://developer.gnome.org/hig/guidelines/keyboard.html
  */
 gboolean
-commonCb_key_press_event(struct Window *window, GdkEvent *event, struct Client *browser)
+commonCb_key_press_event(GdkEvent *event, struct Client *browser)
 {
 	GtkNotebook *notebook = GTK_NOTEBOOK(window->notebook);
 
@@ -84,6 +82,9 @@ commonCb_key_press_event(struct Window *window, GdkEvent *event, struct Client *
 		{
 			switch(((GdkEventKey *)event)->keyval)
 			{
+			case GDK_KEY_Escape:
+				webkit_web_view_terminate_web_process(browser->webView);
+				return TRUE;
 			case GDK_KEY_F4:
 				webkit_web_view_try_close(browser->webView);
 				return TRUE;
@@ -111,7 +112,7 @@ commonCb_key_press_event(struct Window *window, GdkEvent *event, struct Client *
 				return TRUE;
 			case GDK_KEY_p:
 				webkit_print_operation_run_dialog(webkit_print_operation_new(browser->webView),
-				                                  GTK_WINDOW(browser->window->main_window));
+				                                  GTK_WINDOW(window->main_window));
 				return TRUE;
 			}
 		}
@@ -126,7 +127,7 @@ commonCb_key_press_event(struct Window *window, GdkEvent *event, struct Client *
 				gtk_notebook_prev_page(notebook);
 				return TRUE;
 			case GDK_KEY_t:
-				badwolf_new_tab(notebook, new_browser(window, NULL, NULL), TRUE);
+				badwolf_new_tab(notebook, new_browser(NULL, NULL), TRUE);
 				return TRUE;
 			}
 		}
@@ -190,7 +191,7 @@ WebViewCb_key_press_event(WebKitWebView *UNUSED(webView), GdkEvent *event, gpoin
 {
 	struct Client *browser = (struct Client *)user_data;
 
-	if(commonCb_key_press_event(browser->window, event, browser)) return TRUE;
+	if(commonCb_key_press_event(event, browser)) return TRUE;
 
 	return FALSE;
 }
@@ -200,17 +201,17 @@ boxCb_key_press_event(GtkWidget *UNUSED(widget), GdkEvent *event, gpointer user_
 {
 	struct Client *browser = (struct Client *)user_data;
 
-	if(commonCb_key_press_event(browser->window, event, browser)) return TRUE;
+	if(commonCb_key_press_event(event, browser)) return TRUE;
 
 	return FALSE;
 }
 
 gboolean
-main_windowCb_key_press_event(GtkWidget *UNUSED(widget), GdkEvent *event, gpointer user_data)
+main_windowCb_key_press_event(GtkWidget *UNUSED(widget),
+                              GdkEvent *event,
+                              gpointer UNUSED(user_data))
 {
-	struct Window *window = (struct Window *)user_data;
-
-	if(commonCb_key_press_event(window, event, NULL)) return TRUE;
+	if(commonCb_key_press_event(event, NULL)) return TRUE;
 
 	return FALSE;
 }
